/*
 * File: server_fork.c
 * Autore: Iezzi Alessandro
 * Server d'esempio
 */
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdlib.h>
#include <unistd.h>	/* fork */
#include <sys/types.h>	/* pid_t */
/*#include <arpa/inet.h>*/ /* Se si vuole visualizzare l'indirizzo ip del client, decommentare questa riga */
 
#define MAX 8192 /* in bytes, 8KB */
 
int main()
{
	char buff[MAX]; /* dati di invio e ricezione */
	struct sockaddr_in server_addr; /* indirizzo del server */
	struct sockaddr_in client_addr; /* indirizzo del client */
	int sd_server, sd_client; /* i socket descriptor usati per identificare server e client */
 
/*
 * Creazione socket descriptor per il server.
 * AF_INET + SOCK_STREAM --> TCP, utilizzo del protocollo TCP (IPPROTO_TCP)
 */
	if((sd_server = socket(AF_INET, SOCK_STREAM, 0)) < 0)
		printf("Errore nella creazione del server\n");
	
	int optval = 1;
	if(setsockopt(sd_server, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof(optval)) < 0)
		printf("Errore nel riusare la porta");
 
/*
 * Inseriamo nella struttura alcune informazioni
 */
	server_addr.sin_family = AF_INET; /* la famiglia dei protocolli */
	server_addr.sin_port = htons(1745); /* la porta in ascolto */
	server_addr.sin_addr.s_addr = INADDR_ANY; /* dato che  un server bisogna associargli l'indirizzo della macchina su cui sta girando */
 
/*
 * Assegnazione del processo alla socket tramite la funzione BIND
 */
	if(bind(sd_server, (struct sockaddr *)&server_addr, sizeof(server_addr)) < 0)
		printf("Errore di binding\n");
 
/*
 * Si mette in ascolto con un massimo di 20 connessioni
 */
	listen (sd_server, 20);
 
/*
 * Essendo un server monothreading, accetter una sola connessione per volta
 */
	int address_size = sizeof(client_addr); /* dimensione della struttura client_addr */
	pid_t pid; /* Process ID del processo figlio */
	
/* Parte concorrente */
	while(1) {
		sd_client = accept(sd_server, (struct sockaddr *)&client_addr, &address_size);
		if(sd_client < 0) {
			printf("Errore nella chiamata accept\n");
			exit(-1);
		}
		pid = fork();
		
		if(pid == 0) { /* figlio */
			close(sd_server);
			char tmp_string[MAX]; /* servir per capire se chiudere o meno la connessione con il client */
			
			while(strcmp(tmp_string, "EXIT\r\n") != 0) { /* se si riceve il comando EXIT, termina il ciclo */
				memset(buff, '\0', MAX);	/* si cancellano i dati ricevuti in precedenza */
				memset(tmp_string, '\0', MAX);
				recv(sd_client, buff, sizeof(buff), 0);
				strcpy(tmp_string, buff);
				printf("Dati ricevuti: %s", buff);
				strcpy(buff, "Tutto OK!\n");
				send(sd_client, buff, strlen(buff), 0);
			}
			
			close(sd_client); /* disconnessione */
			exit(0); /* chiude il processo */
		} else if(pid > 0) { /* padre */
			close(sd_client); /* si chiude perch non serve pi al padre */
		} else {
			printf("Errore nella creazione del figlio\n");
			exit(-1);
		}
	}
	/* Per terminare l'esecuzione del server  necessario usare i tasti CTRL + C */
 
	return EXIT_SUCCESS;
}